#!/bin/csh -fb
# 
# Mach Operating System
# Copyright (c) 1993 Carnegie Mellon University
# All Rights Reserved.
# 
# Permission to use, copy, modify and distribute this software and its
# documentation is hereby granted, provided that both the copyright
# notice and this permission notice appear in all copies of the
# software, derivative works or modified versions, and any portions
# thereof, and that both notices appear in supporting documentation.
# 
# CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS 
# CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
# ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
# 
# Carnegie Mellon requests users of this software to return to
# 
#  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
#  School of Computer Science
#  Carnegie Mellon University
#  Pittsburgh PA 15213-3890
# 
# any improvements or extensions that they make and grant Carnegie the
# rights to redistribute these changes.
# 
# 
# HISTORY
# $Log:	bootgdb.csh,v $
# Revision 2.1.2.2  93/04/16  17:14:16  pds
# 	Removed install phase, which is now handled by odemake.
# 	Added copyright notice.
# 	[93/04/16            pds]
# 
# Revision 2.1.2.1  93/04/08  13:45:04  pds
# 	Added whist markers.
# 	[93/04/08            pds]
# 
#
# Bootstrap script for building GDB.
#
# Expected arguments:
# $1 - object tree base
# $2 - release tree base
# $3 - source directory
# $4 - GDB target name
# $5 - GDB host name
#
# unset potentially obnoxious make variables
echo Unsetenv-ing MAKEFLAGS OBJECTDIR RELEASEBASE SOURCEBASE VPATH.
unsetenv MAKEFLAGS OBJECTDIR RELEASEBASE SOURCEBASE VPATH
#
echo Bootstrapping GDB.
#
set odir = $1
set rdir = $2
set sdir = $3
set tconf = $4
set hconf = $5
#
echo Configuring GDB in user/gnu for host $hconf and target $tconf.
#
# Check for directory existence
#
if ( ! -d $odir ) then
    echo Cannot access $odir.
    exit 1
endif
#
# Make sure the release directory exists.
#
foreach d ( bin man )
    if ( ! -d $rdir/$d ) then
	echo Cannot access $rdir/$d.
	exit 1
    endif
end
#
# remove dot from the path
# shell help from karl@cs.cmu.edu
# 
@ i = $#path
while ($i > 0)
	if ($path[$i] == .) then
	        echo \[ removing . from path \]
		if ($i == 1) then
			set path=($path[2-])
		else if ($i == $#path) then
			@ i --
			set path=($path[1-$i])
		     else
			@ i = $i - 1
			@ j= $i + 2
			set path=($path[1-$i] $path[$j-])
		endif
	endif
@ i -- 
end
#
# Check configuration.
#
cd $odir
echo Checking configuration in $odir.
if ( ! -e INSTALLDIR ) then
    touch INSTALLDIR
endif
set configureflag = "--srcdir=$sdir --host=$hconf --target=$tconf --prefix=$rdir"
echo $configureflag > INSTALLDIR.new
cmp INSTALLDIR.new INSTALLDIR >& /dev/null
if ( $status != 0 ) then
    echo Configuration has changed - cleaning.
    rm -rf *
    echo $configureflag > INSTALLDIR.new
    echo Configuring host $hconf and target $tconf \[ output to gdb-config \]
    $sdir/configure $configureflag > gdb-config
    if ( $status != 0 ) then
	echo Configuration failed - aborting.
	exit 1
    endif
endif
mv -f INSTALLDIR.new INSTALLDIR
#
# Build GDB and its tools.
#
echo Making GDB with CFLAGS=-g CC=gcc.
$rdir/bin/gnumake -k -j3 CFLAGS=-g CC=gcc
