/* Low level interface to I386 running Mach 3.0.
   Copyright (C) 1992, 1993 Free Software Foundation, Inc.

This file is part of GDB.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/*
 * HISTORY
 * $Log:	i386mach3-tdep.c,v $
 * Revision 2.1.1.1  93/04/16  16:35:19  pds
 * 	Updated copyright and added whist markers.
 * 	[93/04/16            pds]
 * 
 */

#include "defs.h"
#include "inferior.h"

#include <stdio.h>

#include <mach.h>
#include <mach/message.h>
#include <mach/exception.h>
#include <mach_error.h>

/*
 * This is needed for the FP_XXX symbols that indicate the type of floating
 * point support.
 */
#include <mach/i386/fp_reg.h>

/* Hmmm... Should this not be here?
 * Now for i386_float_info() target_has_execution
 */
#include <target.h>

/* This mess is duplicated in bfd/i386mach3.h
 *
 * This is an ugly way to hack around the incorrect
 * definition of UPAGES in i386/machparam.h.
 *
 * The definition should specify the size reserved
 * for "struct user" in core files in PAGES,
 * but instead it gives it in 512-byte core-clicks
 * for i386 and i860.
 */
#include <sys/param.h>
#if UPAGES == 16
#define UAREA_SIZE ctob(UPAGES)
#elif UPAGES == 2
#define UAREA_SIZE (NBPG*UPAGES)
#else
FIXME!! UPAGES is neither 2 nor 16
#endif

/* @@@ Should move print_387_status() to i387-tdep.c */
extern void print_387_control_word ();		/* i387-tdep.h */
extern void print_387_status_word ();

extern struct ext_format ext_format_i387;

#ifndef	private
#define private static
#endif

/* Find offsets to thread states at compile time.
 * If your compiler does not grok this, calculate offsets
 * offsets yourself and use them (or get a compatible compiler :-)
 */

#define  REG_OFFSET(reg) (int)(&((struct i386_thread_state *)0)->reg)

/*
 * reg_offset[N] is the offset in a struct i386_thread_state of the
 * value for GDB's register N.
 */

static int reg_offset[] = 
{
  REG_OFFSET(eax),  REG_OFFSET(ecx), REG_OFFSET(edx), REG_OFFSET(ebx),
  REG_OFFSET(uesp), REG_OFFSET(ebp), REG_OFFSET(esi), REG_OFFSET(edi),
  REG_OFFSET(eip),  REG_OFFSET(efl), REG_OFFSET(cs),  REG_OFFSET(ss),
  REG_OFFSET(ds),   REG_OFFSET(es),  REG_OFFSET(fs),  REG_OFFSET(gs)
};

#define REG_ADDRESS(state,regnum) ((char *)(state)+reg_offset[regnum])

/*
 * Fetch inferiors registers for gdb.
 * REGNO specifies which (as gdb views it) register, -1 for all.
 */
void
mach3_fetch_registers (regno)
     int	regno;
{
  kern_return_t			kr;
  int				i;
  thread_state_data_t		state;
  mach_msg_type_number_t	stateCnt = i386_THREAD_STATE_COUNT;
  
  if (! MACH_PORT_VALID (current_thread))
    error ("fetch inferior registers: Invalid thread");

  kr = thread_get_state (current_thread, i386_THREAD_STATE, state, &stateCnt);
  if (kr == KERN_SUCCESS)
    {
      /*
       * This has to be done one-by-one, becuase Mach and GDB do not store
       * the registers in the same order.
       */
      for (i = 0; i < NUM_REGS; i++)
	{
	  supply_register (i, REG_ADDRESS (state, i));
	  register_valid[i] = TRUE;
	}
    }
  else
    message ("Unable to fetch thread state for 0x%x: %s.", current_thread,
	     mach_error_string (kr));
}

/* Store the register values back into the inferior.
 * mach3_prepare_to_store guarantees that all registers are valid when
 * entering here.
 */
void
mach3_store_registers (regno)
     int	regno;
{
  kern_return_t			kr;
  int				i;
  thread_state_data_t		state;
  mach_msg_type_number_t	stateCnt = i386_THREAD_STATE_COUNT;

  if (! MACH_PORT_VALID (current_thread))
    error ("store inferior registers: Invalid thread");

  /*
   * This has to be done one-by-one, becuase Mach and GDB do not store
   * the registers in the same order.
   */
  for (i = 0; i < NUM_REGS; i++)
    memcpy (REG_ADDRESS (state, i), &registers[REGISTER_BYTE (i)],
	    REGISTER_RAW_SIZE (i));
  
  /*
   * Write gdb's current view of the registers to the thread.
   */
  kr = thread_set_state (current_thread, i386_THREAD_STATE, state,
			 i386_THREAD_STATE_COUNT);
  
  if (kr != KERN_SUCCESS)
    message ("Unable to store thread state for 0x%x: %s", current_thread,
	     mach_error_string (kr));
}

boolean_t
mach3_fetch_user_state (sp, usp_ptr, ufp_ptr, upc_ptr)
     CORE_ADDR	sp;
     CORE_ADDR *usp_ptr;
     CORE_ADDR *ufp_ptr;
     CORE_ADDR *upc_ptr;
{
  if (EMULATOR_BASE > sp || sp >= EMULATOR_END)
    return FALSE;

  sp = (sp | (EMULATOR_STACK_SIZE - 1)) + 1 - sizeof(struct emul_stack_top);

  if (target_read_memory (sp, (char *) usp_ptr, sizeof(*usp_ptr)))
    {
      message ("Can't read user SP from emulator stack address 0x%x.", sp);
      return FALSE;
    }
  SWAP_TARGET_AND_HOST((char *) usp_ptr, sizeof(*usp_ptr));

  /*
   * The emulator saves EBX, ESI, EDI, EBP, and ESP on the emulator stack
   * beneath the user SP.
   */
  sp -= (4 * JB_ELEMENT_SIZE);
  if (target_read_memory (sp, (char *) ufp_ptr, sizeof(*ufp_ptr)))
    {
      message ("Can't read user FP from emulator stack address 0x%x.", sp);
      return FALSE;
    }
  SWAP_TARGET_AND_HOST((char *) ufp_ptr, sizeof(*ufp_ptr));

  /*
   * The user PC is saved on the user's stack beneath saved flags and
   * EAX, ECX, EDX.
   */
  sp = *usp_ptr - (4 * JB_ELEMENT_SIZE);
  if (target_read_memory (sp, (char *) upc_ptr, sizeof(*upc_ptr)))
    {
      message ("Can't read user PC from stack address 0x%x.", sp);
      return FALSE;
    }
  SWAP_TARGET_AND_HOST((char *) upc_ptr, sizeof(*upc_ptr));

  return TRUE;
}

/* Return the address in the core dump or inferior of register REGNO.
 * BLOCKEND should be the address of the end of the UPAGES area read
 * in memory, but it's not?
 *
 * Currently our UX server dumps the whole thread state to the
 * core file. If your UX does something else, adapt the routine
 * below to return the offset to the given register.
 * 
 * Called by coredep.c(fetch_core_registers)
 */

unsigned int
register_addr (regno, blockend)
     int regno;
     int blockend;
{
  unsigned int addr;

  if (regno < 0 || regno >= NUM_REGS)
    error ("Invalid register number %d.", regno);

  /* UAREA_SIZE == 8 kB in i386 */
  addr = (unsigned int)REG_ADDRESS (UAREA_SIZE - sizeof(struct i386_thread_state), regno);

  return addr;
}

/*
 * jtv@hut.fi: I copied and modified this 387 code from gdb/i386-xdep.c.
 *
 * i387 status dumper. See also i387-tdep.c.  Except for the OS dependency
 * of the structure naming this routine could be moved to i387-tdep.c.
 */

#define	NUM_FP_REGS	8
#define	SIZEOF_FP_REG	10

/* 
 * Print out the i387 status structures.
 * The registers are declared as 'char *', because the Mach 3.0 header
 * file declaration is bogus.
 */
private void
print_387_status (fp_save, fp_regs)
     struct i386_fp_save *  fp_save;
     char *		    fp_regs;
{
  int		i;
  int		fpreg;
  int		top;
  
  if (fp_save->fp_status != 0)
    print_387_status_word (fp_save->fp_status);
  
  print_387_control_word (fp_save->fp_control);
  printf ("last exception: ");
  printf ("opcode %s; ", local_hex_string(fp_save->fp_opcode));
  printf ("pc %s:", local_hex_string(fp_save->fp_cs));
  printf ("%s; ", local_hex_string(fp_save->fp_eip));
  printf ("operand %s", local_hex_string(fp_save->fp_ds));
  printf (":%s\n", local_hex_string(fp_save->fp_dp));
  
  top = (fp_save->fp_status >> 11) & 7;
  
  /*
   * Since we are printing the registers in reverse order, move
   * the pointer to the end of the block.
   */
  fp_regs += NUM_FP_REGS * SIZEOF_FP_REG;

  printf ("regno  tag  msb              lsb  value\n");
  for (fpreg = 7; fpreg >= 0; fpreg--)
    {
      double val;
      
      printf ("%s %d: ", fpreg == top ? "=>" : "  ", fpreg);
      
      switch ((fp_save->fp_tag >> (fpreg * 2)) & 3) 
	{
	case 0: printf ("valid "); break;
	case 1: printf ("zero  "); break;
	case 2: printf ("trap  "); break;
	case 3: printf ("empty "); break;
	}
      for (i = 9; i >= 0; i--)
	printf ("%02x", --fp_regs);
      
      ieee_extended_to_double (&ext_format_i387, fp_regs, &val);
      printf ("  %g\n", val);
    }
  if (fp_save->fp_unused_1)
    warning ("fp_unused_1 is %s.", local_hex_string(fp_save->fp_unused_1));
  if (fp_save->fp_unused_2)
    warning ("fp_unused_2 is %s.", local_hex_string(fp_save->fp_unused_2));
  if (fp_save->fp_unused_3)
    warning ("fp_unused_3 is %s.", local_hex_string(fp_save->fp_unused_3));
  if (fp_save->fp_unused_4)
    warning ("fp_unused_4 is %s.", local_hex_string(fp_save->fp_unused_4));
}
	

/* Mach 3 specific routines.
 */
private boolean_t
get_i387_state (fp_save, fp_regs)
     struct i386_fp_save *	fp_save;
     char *			fp_regs;
{
  kern_return_t			kr;
  thread_state_data_t		state;
  mach_msg_type_number_t	fsCnt = i386_FLOAT_STATE_COUNT;
  struct i386_float_state *	fsp;
  
  kr = thread_get_state (current_thread, i386_FLOAT_STATE, state, &fsCnt);

  if (kr != KERN_SUCCESS)
    {
      message ("Can not get floating point state for thread 0x%x: %s",
	       current_thread, mach_error_string (kr));
      return FALSE;
    }

  fsp = (struct i386_float_state *)state;
  /*
   * The 387 chip (also 486 counts) or a software emulator?
   */
  if (! fsp->initialized || (fsp->fpkind != FP_387 && fsp->fpkind != FP_SOFT))
    return FALSE;

  *fp_save = *((struct i386_fp_save *) fsp->hw_state);

  memcpy (fp_regs, fsp->hw_state + sizeof(struct i386_float_state),
	  NUM_FP_REGS * SIZEOF_FP_REG);

  return TRUE;
}

private boolean_t
get_i387_core_state (fp_save, fp_regs)
     struct i386_fp_save *	fp_save;
     char *			fp_regs;
{
  /* Not implemented yet. Core files do not contain float state. */
  return FALSE;
}

/*
 * This is called by "info float" command
 */
void
i386_mach3_float_info()
{
  struct i386_float_state	fp_save;
  char				fp_regs[NUM_FP_REGS * SIZEOF_FP_REG];
  boolean_t			valid = FALSE;
  
  if (target_has_execution)
    valid = get_i387_state (&fp_save, fp_regs);
#if 0  
  else if (WE HAVE CORE FILE)  /* @@@@ Core files not supported */
    valid = get_i387_core_state (&fp_save, fp_regs);
#endif    

  if (! valid) 
    {
      message("no floating point status saved");
      return;
    }

  print_387_status (&fp_save, fp_regs);
}
