/* Target-dependent code for M88000 architecture machines running Mach 3.0.
   Copyright (C) 1993 Free Software Foundation, Inc.
   Contributed by Jeffrey Friedl(jfriedl@omron.co.jp) and
   Peter Stout(pds@cs.cmu.edu).

This file is part of GDB.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/*
 * HISTORY
 * $Log:	m88kmach3-tdep.c,v $
 * Revision 2.1.1.1  93/04/16  15:51:32  pds
 * 	Created.
 * 
 */

#include "defs.h"
#include "inferior.h"

#include <stdio.h>

#include <mach.h>
#include <mach/message.h>
#include <mach/exception.h>
#include <mach_error.h>

#define	REG_OFFSET(reg)	(int)(&((struct luna88k_thread_state *)0)->reg)

/*
 * At reg_offset[i] is the offset to the luna88k_thread_state location where
 * the GDB registers[i] is stored.
 */

static int reg_offset[] =
{
    REG_OFFSET(r[0]), REG_OFFSET(r[1]), REG_OFFSET(r[2]), REG_OFFSET(r[3]),
    REG_OFFSET(r[4]), REG_OFFSET(r[5]), REG_OFFSET(r[6]), REG_OFFSET(r[7]),
    REG_OFFSET(r[8]), REG_OFFSET(r[9]), REG_OFFSET(r[10]), REG_OFFSET(r[11]),
    REG_OFFSET(r[12]), REG_OFFSET(r[13]), REG_OFFSET(r[14]), REG_OFFSET(r[15]),
    REG_OFFSET(r[16]), REG_OFFSET(r[17]), REG_OFFSET(r[18]), REG_OFFSET(r[19]),
    REG_OFFSET(r[20]), REG_OFFSET(r[21]), REG_OFFSET(r[22]), REG_OFFSET(r[23]),
    REG_OFFSET(r[24]), REG_OFFSET(r[25]), REG_OFFSET(r[26]), REG_OFFSET(r[27]),
    REG_OFFSET(r[280]), REG_OFFSET(r[29]), REG_OFFSET(r[30]), REG_OFFSET(r[32]),
    REG_OFFSET(psr), REG_OFFSET(fpsr), REG_OFFSET(fpcr), REG_OFFSET(sxip),
    REG_OFFSET(snip), REG_OFFSET(sfip)
};

#define	ZERO_REGS(regnum, count)					    \
  do {									    \
    const int _r = (regnum);						    \
    int _c = (count);							    \
									    \
    memset (&registers[REGISTER_BYTE (_r)], 0,				    \
	    _c * REGISTER_RAW_SIZE (_r));				    \
    while (_c > 0)							    \
      register_valid[_r + --_c] = TRUE;					    \
  } while (0)

#define	FETCH_REGS(state, regnum, count)				    \
  do {									    \
    const int _r = (regnum);						    \
    int _c = (count);							    \
									    \
    memcpy (&registers[REGISTER_BYTE (_r)],				    \
	    ((char *) (state)) + reg_offset[_r],			    \
	    _c * REGISTER_RAW_SIZE (_r));				    \
    while (_c > 0)							    \
      register_valid[_r + --_c] = TRUE;					    \
  } while (0)

#define STORE_REGS(state, regnum, count)				    \
  do {									    \
    const int _r = (regnum);						    \
									    \
    memcpy (((char *) (state)) + reg_offset[_r],			    \
	    &registers[REGISTER_BYTE (_r)],				    \
	    (count) * REGISTER_RAW_SIZE (_r));				    \
  } while (0)

/*
 * Fetch thread's registers. if regno == -1, fetch all regs.
 */
void
mach3_fetch_registers (regno)
     int regno;
{
  kern_return_t			kr;
  mach_msg_type_number_t	stateCnt;
  thread_state_data_t		state;
  struct luna88k_exc_state	exc_state;

  if (! MACH_PORT_VALID (current_thread))
    error ("fetch inferior registers: Invalid thread");
  
  if (regno < -1 || regno >= NUM_REGS)
    error ("invalid register %d supplied to fetch_inferior_registers",regno);

  stateCnt = LUNA88K_THREAD_STATE_COUNT;
  kr = thread_get_state (current_thread, LUNA88K_THREAD_STATE, state,
			 &stateCnt);
  if (kr != KERN_SUCCESS)
    error ("Unable to fetch thread state for thread 0x%x: %s.",
	   current_thread, mach_error_string (kr));

  /*
   * As long as we are fetching the register state, we might as
   * well validate all of the registers we fetched.
   */

  /* ZERO_REGNUM is always zero */
  ZERO_REGS (0, 1);

  /*
   * General Purpose Registers.
   */
  FETCH_REGS (state, SRP_REGNUM, 31);

  /*
   * Supply Floating Point Control Registers
   */
  FETCH_REGS (state, FPSR_REGNUM, 2);

  /*
   * Program Status Register
   */
  FETCH_REGS (state, PSR_REGNUM, 1);

  /*
   * Supply PCs.
   */
  FETCH_REGS (state, PC_REGNUM, 3);
}

/* Store gdb's view of registers to the thread.
 * mach3_prepare_to_store() guarantees that all registers are valid when
 * entering here.
 *
 * Hmm. It seems that gdb set $reg=value command first reads everything,
 * then sets the reg and then stores everything. -> we must make sure
 * that the immutable registers are not changed.  This is currently
 * guaranteed by the Mach 3.0 interface not allowing us to set any
 * of those registers.
 */

void
mach3_store_registers (regno)
     register int regno;
{
  thread_state_data_t state;
  kern_return_t kr;
  
  if (! MACH_PORT_VALID (current_thread))
    error ("store inferior registers: Invalid thread");
  
  /*
   * Check for read only regs.
   */
  if (regno == 0)
    {
      message ("You can not alter read-only register `%s'",reg_names[regno]);
      return;
    }

  /*
   * General Purpose Registers.
   */
  STORE_REGS (state, 1, 31);

  /*
   * Save Floating Point Control Registers.
   */
  STORE_REGS (state, FPSR_REGNUM, 2);

  /*
   * Save PCs.
   */
  STORE_REGS (state, PC_REGNUM, 3);

  /*
   * Save Program Status Register.
   */
  STORE_REGS (state, PSR_REGNUM, 1);

  kr = thread_set_state (current_thread, LUNA88K_THREAD_STATE, state,
			 LUNA88K_THREAD_STATE_COUNT);
  if (kr != KERN_SUCCESS)
    error ("Unable to store thread state for thread 0x%x: %s.",
	   current_thread, mach_error_string (kr));
}

/*
 * Figure out where the longjmp will land.
 * We expect the first argument to be a pointer to the jmp_buf structure
 * from which we extract the PC at which we will land.  The PC is the
 * first word in the jmp_buf.
 */

int
mach3_get_longjmp_target (pc_ptr)
     CORE_ADDR *pc_ptr;
{
  CORE_ADDR	jb_addr;

  jb_addr = read_register (RV_REGNUM);

  if (target_read_memory (jb_addr, pc_ptr, sizeof(*pc_ptr)))
    return 0;

  SWAP_TARGET_AND_HOST (pc_ptr, sizeof(*pc_ptr));

  return 1;
}

/*
 * Current emulator always saves the USP on top of the emulator stack,
 * below the struct emul_stack_top stuff.
 * 
 * NOTE:  The definitions for SC_SP and SC_PC are based on examining
 * the definition of a struct sigcontext in signal.h
 */
#define	SC_SP	2
#define	SC_PC	7
#define	SC_ELEMENT_SIZE	4

boolean_t
mach3_fetch_user_state (sp, usp_ptr, ufp_ptr, upc_ptr)
     CORE_ADDR	sp;
     CORE_ADDR *usp_ptr;
     CORE_ADDR *ufp_ptr;
     CORE_ADDR *upc_ptr;
{
  CORE_ADDR sc_ptr;

  if (EMULATOR_BASE > sp || sp >= EMULATOR_END)
    return FALSE;

  sp = (sp | (EMULATOR_STACK_SIZE - 1)) + 1 - sizeof(struct emul_stack_top);

  if (target_read_memory (sp, (char *) &sc_ptr, sizeof(sc_ptr)))
    {
      message ("Can't read sigcontext ptr from emulator stack address 0x%x",
	       sp);
      return FALSE;
    }
  SWAP_TARGET_AND_HOST((char *) &sc_ptr, sizeof(sc_ptr));

  sp = sc_ptr + SC_SP * SC_ELEMENT_SIZE;
  if (target_read_memory (sp, (char *) usp_ptr, sizeof(*usp_ptr)))
    {
      message ("Can't read user SP from stack address 0x%x.", sp);
      return FALSE;
    }
  SWAP_TARGET_AND_HOST((char *) usp_ptr, sizeof(*usp_ptr));

  sp = sc_ptr + SC_PC * SC_ELEMENT_SIZE;
  if (target_read_memory (sp, (char *) upc_ptr, sizeof(*upc_ptr)))
    {
      message ("Can't read user PC from stack address 0x%x.", sp);
      return FALSE;
    }
  SWAP_TARGET_AND_HOST((char *) upc_ptr, sizeof(*upc_ptr));

  /*
   * We are expecting the backtrace code to fix up the FP for us.
   */
  *ufp_ptr = 0;

  return TRUE;
}
