/* Target-dependent code for MIPS architecture machines running Mach 3.0.
   Copyright (C) 1992, 1993 Free Software Foundation, Inc.
   Contributed by Jukka Virtanen(jtv@hut.fi) and Peter Stout(pds@cs.cmu.edu).

This file is part of GDB.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/*
 * HISTORY
 * $Log:	mipsmach3-tdep.c,v $
 * Revision 2.1.1.1  93/04/16  16:35:27  pds
 * 	Added whist markers.
 * 	[93/04/16            pds]
 * 
 */

#include "defs.h"
#include "inferior.h"

#include <stdio.h>

#include <mach.h>
#include <mach/message.h>
#include <mach/exception.h>
#include <mach_error.h>

/* Find offsets to thread states at compile time.
 * If your compiler does not grok this, check the hand coded
 * offsets and use them.
 */

#if 1

#define  REG_OFFSET(reg) (int)(&((struct mips_thread_state *)0)->reg)
#define CREG_OFFSET(reg) (int)(&((struct mips_float_state *)0)->reg)
#define EREG_OFFSET(reg) (int)(&((struct mips_exc_state *)0)->reg)

/* at reg_offset[i] is the offset to the mips_thread_state
 * location where the gdb registers[i] is stored.
 *
 * -1 means mach does not save it anywhere.
 */
static int reg_offset[] = 
{
  /*  zero		at		  v0		    v1       */
      -1,           REG_OFFSET(r1),   REG_OFFSET(r2),   REG_OFFSET(r3),

  /*  a0		a1		  a2		    a3       */
  REG_OFFSET(r4),   REG_OFFSET(r5),   REG_OFFSET(r6),   REG_OFFSET(r7),

  /*  t0		t1		  t2		    t3       */
  REG_OFFSET(r8),   REG_OFFSET(r9),   REG_OFFSET(r10),  REG_OFFSET(r11),

  /*  t4		t5		  t6		    t7       */
  REG_OFFSET(r12),  REG_OFFSET(r13),  REG_OFFSET(r14),  REG_OFFSET(r15),

  /*  s0		s1		  s2		    s3       */
  REG_OFFSET(r16),  REG_OFFSET(r17),  REG_OFFSET(r18),  REG_OFFSET(r19),

  /*  s4		s5		  s6		    s7       */
  REG_OFFSET(r20),  REG_OFFSET(r21),  REG_OFFSET(r22),  REG_OFFSET(r23),

  /*  t8		t9		  k0		    k1       */
  REG_OFFSET(r24),  REG_OFFSET(r25),  REG_OFFSET(r26),  REG_OFFSET(r27),

  /*  gp		sp	      s8(30) == fp(72)	    ra       */
  REG_OFFSET(r28),  REG_OFFSET(r29),  REG_OFFSET(r30),  REG_OFFSET(r31),

  /*  sr(32) PS_REGNUM   */
  EREG_OFFSET(coproc_state),

  /*  lo(33)		hi(34)    */
  REG_OFFSET(mdlo), REG_OFFSET(mdhi),

  /*  bad(35)          	      cause(36)	         pc(37)  */
  EREG_OFFSET(address),  EREG_OFFSET(cause), REG_OFFSET(pc),

  /*  f0(38)		 f1(39)		    f2(40)	       f3(41)   */
  CREG_OFFSET(r0),   CREG_OFFSET(r1),   CREG_OFFSET(r2),   CREG_OFFSET(r3),
  CREG_OFFSET(r4),   CREG_OFFSET(r5),   CREG_OFFSET(r6),   CREG_OFFSET(r7),
  CREG_OFFSET(r8),   CREG_OFFSET(r9),   CREG_OFFSET(r10),  CREG_OFFSET(r11),
  CREG_OFFSET(r12),  CREG_OFFSET(r13),  CREG_OFFSET(r14),  CREG_OFFSET(r15),
  CREG_OFFSET(r16),  CREG_OFFSET(r17),  CREG_OFFSET(r18),  CREG_OFFSET(r19),
  CREG_OFFSET(r20),  CREG_OFFSET(r21),  CREG_OFFSET(r22),  CREG_OFFSET(r23),
  CREG_OFFSET(r24),  CREG_OFFSET(r25),  CREG_OFFSET(r26),  CREG_OFFSET(r27),
  CREG_OFFSET(r28),  CREG_OFFSET(r29),  CREG_OFFSET(r30),  CREG_OFFSET(r31),

  /*  fsr(70)		fir(71)		fp(72) == s8(30) */
  CREG_OFFSET(csr),  CREG_OFFSET(esr),  REG_OFFSET(r30)
};
#else
/* If the compiler does not grok the above defines */
static int reg_offset[] = 
{
/* mach_thread_state offsets: */
  -1,  0,  4,  8,  12, 16, 20, 24,  28, 32, 36, 40,  44, 48, 52, 56,
  60, 64, 68, 72,  76, 80, 84, 88,  92, 96,100,104, 108,112,116,120,
/*sr, lo, hi,addr,cause,pc   */
   8,124,128,  4,   0,132,
/* mach_float_state offsets: */
   0,  4,  8, 12,  16, 20, 24, 28,  32, 36, 40, 44,  48, 52, 56, 60,
  64, 68, 72, 76,  80, 84, 88, 92,  96,100,104,108, 112,116,120,124,
/*fsr,fir*/
 128,132,
/* FP_REGNUM pseudo maps to s8==r30 in mach_thread_state */
 116
};
#endif

/*
 * Fetch/Store COUNT contiguous registers from/to thread STATE starting
 * with REGNUM.  The caller knows that the registers handled in one
 * transaction are consecutive and the same size.
 */
#define	ZERO_REGS(regnum, count)					    \
  do {									    \
    const int _r = (regnum);						    \
    int _c = (count);							    \
									    \
    memset (&registers[REGISTER_BYTE (_r)], 0,				    \
	    _c * REGISTER_RAW_SIZE (_r));				    \
    while (_c > 0)							    \
      register_valid[_r + --_c] = TRUE;					    \
  } while (0)
    

#define	FETCH_REGS(state, regnum, count)				    \
  do {									    \
    const int _r = (regnum);						    \
    int _c = (count);							    \
									    \
    memcpy (&registers[REGISTER_BYTE (_r)],				    \
	    ((char *) (state)) + reg_offset[_r],			    \
	    _c * REGISTER_RAW_SIZE (_r));				    \
    while (_c > 0)							    \
      register_valid[_r + --_c] = TRUE;					    \
  } while (0)

#define STORE_REGS(state, regnum, count)				    \
  do {									    \
    const int _r = (regnum);						    \
									    \
    memcpy (((char *) (state)) + reg_offset[_r],			    \
	    &registers[REGISTER_BYTE (_r)],				    \
	    (count) * REGISTER_RAW_SIZE (_r));				    \
  } while (0)

#define REGS_ALL    -1
#define REGS_NORMAL  1
#define REGS_EXC     2
#define REGS_COP1    4

/* Hardware regs that matches FP_REGNUM */
#define MACH_FP_REGNUM 30

/* Fech thread's registers. if regno == -1, fetch all regs */
void
mach3_fetch_registers (regno)
     int regno;
{
  kern_return_t			kr;
  mach_msg_type_number_t	stateCnt;
  thread_state_data_t		state;
  struct mips_exc_state		exc_state;

  int which_regs = 0; /* A bit mask */

  if (! MACH_PORT_VALID (current_thread))
    error ("fetch inferior registers: Invalid thread");
  
  if (regno < -1 || regno >= NUM_REGS)
    error ("invalid register %d supplied to fetch_inferior_registers", regno);

  if (regno == -1)
    which_regs = REGS_ALL;
  else if (regno == ZERO_REGNUM)
    {
      ZERO_REGS (ZERO_REGNUM, 1);
      register_valid[ZERO_REGNUM] = TRUE;
      return;
    }
  else if ((ZERO_REGNUM < regno && regno < PS_REGNUM)
	   || regno == FP_REGNUM
	   || regno == LO_REGNUM
	   || regno == HI_REGNUM
	   || regno == PC_REGNUM)
    which_regs = REGS_NORMAL;
  else if (FP0_REGNUM <= regno && regno <= FCRIR_REGNUM)
    which_regs = REGS_COP1 | REGS_EXC;
  else
    which_regs = REGS_EXC;

  /* fetch regs saved to mips_thread_state */
  if (which_regs & REGS_NORMAL)
    {
      stateCnt = MIPS_THREAD_STATE_COUNT;
      kr = thread_get_state (current_thread, MIPS_THREAD_STATE, state,
			     &stateCnt);
      if (kr != KERN_SUCCESS)
	error ("Unable to fetch thread state for thread 0x%x: %s.",
	       current_thread, mach_error_string (kr));

      /*
       * As long as we are fetching the register state, we might as
       * well validate all of the registers we fetched.
       */

      /* ZERO_REGNUM is always zero */
      ZERO_REGS (ZERO_REGNUM, 1);
      
      /*
       * General Purpose Registers.
       */
      FETCH_REGS (state, ZERO_REGNUM + 1, 31);
      
      /*
       * Supply mulitply/divide temporaries.
       */
      FETCH_REGS (state, LO_REGNUM, 2);

      /*
       * Supply PC.
       */
      supply_register (PC_REGNUM, ((char *) state) + reg_offset[PC_REGNUM]);

      /*
       * Mach 3.0 saves FP to MACH_FP_REGNUM.
       * For some reason gdb wants to assign a pseudo register for it.
       */
      /*
       * See comment in below in mach3_fetch_user_state and in
       * fetch_inferior_registers in mips-nat.c.
       */
      supply_register (FP_REGNUM, ((char *)state) + reg_offset[MACH_FP_REGNUM]);
    }

  /* Read exc state. Also read if need to fetch floats */
  if (which_regs & REGS_EXC)
    {
      stateCnt = MIPS_EXC_STATE_COUNT;
      kr = thread_get_state (current_thread, MIPS_EXC_STATE,
			     (thread_state_t) &exc_state, &stateCnt);
      if (kr != KERN_SUCCESS)
	error ("Unable to fetch co-processor state for thread 0x%x: %s.",
	       current_thread, mach_error_string (kr));

      /*
       * We need to fetch exc_state to see if the floating
       * state is valid for the thread.
       */

      /* cproc_state: Which coprocessors the thread uses */
      supply_register (PS_REGNUM, ((char *)&exc_state) + reg_offset[PS_REGNUM]);
      supply_register (BADVADDR_REGNUM,
		       ((char *) &exc_state) + reg_offset[BADVADDR_REGNUM]);
      supply_register (CAUSE_REGNUM,
		       ((char *) &exc_state) + reg_offset[CAUSE_REGNUM]);
    }


  if (which_regs & REGS_COP1)
    if (! (exc_state.coproc_state & MIPS_STATUS_USE_COP1))
      ZERO_REGS (FP0_REGNUM, 34);
    else
      {
	stateCnt = MIPS_FLOAT_STATE_COUNT;
	kr = thread_get_state (current_thread, MIPS_FLOAT_STATE, state,
			       &stateCnt);
	if (kr != KERN_SUCCESS)
	  error ("Unable to fetch FP-state for thread 0x%x: %s.",
		 current_thread, mach_error_string (kr));
	FETCH_REGS (state, FP0_REGNUM, 34);
      }
}

/* Store gdb's view of registers to the thread.
 * mach3_prepare_to_store() guarantees that all registers are valid when
 * entering here.
 *
 * Hmm. It seems that gdb set $reg=value command first reads everything,
 * then sets the reg and then stores everything. -> we must make sure
 * that the immutable registers are not changed.  This is currently
 * guaranteed by the Mach 3.0 interface not allowing us to set any
 * of those registers.
 */

void
mach3_store_registers (regno)
     register int regno;
{
  thread_state_data_t state;
  kern_return_t kr;
  
  if (! MACH_PORT_VALID (current_thread))
    error ("store inferior registers: Invalid thread");
  
  /*
   * Check for read only regs.
   */
  if (regno == ZERO_REGNUM	||
      regno == PS_REGNUM	||
      regno == BADVADDR_REGNUM	||
      regno == CAUSE_REGNUM	||
      regno == FCRIR_REGNUM)
    {
      message ("You can not alter read-only register `%s'", reg_names[ regno ]);
      return;
    }

  if (regno == -1 || (ZERO_REGNUM < regno && regno <= PC_REGNUM))
    {
#if 1
      /* Mach 3.0 saves thread's FP to MACH_FP_REGNUM.
       * GDB wants assigns a pseudo register FP_REGNUM for frame pointer.
       *
       * @@@ Here I assume (!) that gdb's FP has the value that
       *     should go to threads frame pointer. If not true, this
       *     fails badly!!!!!
       */
      /*
       * See comment in below in mach3_fetch_user_state and in
       * fetch_inferior_registers in mips-nat.c.
       */
      memcpy (&registers[REGISTER_BYTE (MACH_FP_REGNUM)],
	     &registers[REGISTER_BYTE (FP_REGNUM)],
	     REGISTER_RAW_SIZE (FP_REGNUM));
#endif
      
      /*
       * General purpose registers.
       */
      STORE_REGS (state, 1, 31);

      /* Save mdlo, mdhi */
      STORE_REGS (state, LO_REGNUM, 2);

     /* Save PC */
      STORE_REGS (state, PC_REGNUM, 1);

      kr = thread_set_state (current_thread, MIPS_THREAD_STATE, state,
			     MIPS_THREAD_STATE_COUNT);
      if (kr != KERN_SUCCESS)
	error ("Unable to store thread state for thread 0x%x: %s.",
	       current_thread, mach_error_string (kr));
    }
  
  if (regno == -1 || regno >= FP0_REGNUM)
    {
      /* If thread has floating state, save it */
      if (read_register (PS_REGNUM) & MIPS_STATUS_USE_COP1)
	{
	  STORE_REGS (state, FP0_REGNUM, 34);
	
	  kr = thread_set_state (current_thread, MIPS_FLOAT_STATE, state,
				 MIPS_FLOAT_STATE_COUNT);
	  if (kr != KERN_SUCCESS)
	    error ("Unable to store floating state for thread 0x%x: %s.",
		   current_thread, mach_error_string (kr));
	}
      else if (regno != -1)
	message
	  ("Thread does not use floating point unit, floating regs not saved.");
    }
}

/*
 * Figure out where the longjmp will land.
 * We expect the first argument to be a pointer to the jmp_buf structure
 * from which we extract the PC (JB_PC) at which we will land.
 *
 * NOTE: The Mach 3.0 setjmp.h does not define the JB_XXX symbols, so we hack.
 */

#define	JB_PC	2
#define	JB_ELEMENT_SIZE	4

int
mach3_get_longjmp_target (pc_ptr)
     CORE_ADDR *pc_ptr;
{
  CORE_ADDR	jb_addr;

  jb_addr = read_register (A0_REGNUM);

  if (target_read_memory (jb_addr + JB_PC * JB_ELEMENT_SIZE, pc_ptr,
			  sizeof(*pc_ptr)))
    return 0;

  SWAP_TARGET_AND_HOST (pc_ptr, sizeof(*pc_ptr));

  return 1;
}

/*
 * Current emulator always saves the USP on top of the emulator stack,
 * below the struct emul_stack_top stuff.
 */
boolean_t
mach3_fetch_user_state (sp, usp_ptr, ufp_ptr, upc_ptr)
     CORE_ADDR	sp;
     CORE_ADDR *usp_ptr;
     CORE_ADDR *ufp_ptr;
     CORE_ADDR *upc_ptr;
{
  if (EMULATOR_BASE > sp || sp >= EMULATOR_END)
    return FALSE;

  sp = (sp | (EMULATOR_STACK_SIZE - 1)) + 1 - sizeof(struct emul_stack_top);

  if (target_read_memory (sp, (char *) usp_ptr, sizeof(*usp_ptr)))
    {
      message ("Can't read user sp from emulator stack address 0x%x", sp);
      return FALSE;
    }
  SWAP_TARGET_AND_HOST((char *) usp_ptr, sizeof(*usp_ptr));

  /*
   * In fetch_inferior_registers(mips-nat.c) it says that the FP must
   * appear to be 0, as it is faked in the stack handling code.  Since,
   * I am not sure how to find the last FP at this point, I am going
   * to hope that 0 works correctly.
   */
  *ufp_ptr = 0;

  sp = *usp_ptr + JB_PC * JB_ELEMENT_SIZE;
  if (target_read_memory (sp, (char *) upc_ptr, sizeof(*upc_ptr)))
    {
      message ("Can't read user PC from stack address 0x%x.", sp);
      return FALSE;
    }
  SWAP_TARGET_AND_HOST((char *) upc_ptr, sizeof(*upc_ptr));

  return TRUE;
}
