/* Mach 3.0 common definitions and global vars.

   Copyright (C) 1992, 1993 Free Software Foundation, Inc.

This file is part of GDB.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/*
 * HISTORY
 * $Log:	os-mach3.h,v $
 * Revision 2.1.1.1  93/04/16  16:35:51  pds
 * 	Added whist markers.
 * 	[93/04/16            pds]
 * 
 */

#ifndef _OS_MACH3_H_
#define _OS_MACH3_H_

#include <mach.h>
#include <servers/machid_types.h>

/* @@@ change to MACH3 or better to remove.
 * However, there are still some places in
 * generic gdb code that use ifdef MACH
 */
#ifndef MACH
#define MACH 1
#endif	MACH

/*
 * Thread port of the current thread in the inferior.
 */
extern thread_t current_thread;

/*
 * Pointer to the child creation routine if there is one.
 */
extern void (*mach3_fork_inferior_func) PARAMS ((char *, char *, char **,
						 void (*)(void),void (*)(int)));


/*
 * Various hooks used by generic parts of GDB to call Mach 3.0 specific code.
 */

extern char *
mach3_output_thread_id PARAMS ((mach_id_t));
#define THREAD_OUTPUT_ID(mid)	mach3_output_thread_id (mid)

extern mach_id_t
mach3_parse_thread_id PARAMS ((char *));
#define THREAD_PARSE_ID(arg)	mach3_parse_thread_id (arg)

extern int
mach3_prepare_to_proceed PARAMS ((void));
#define PREPARE_TO_PROCEED()	mach3_prepare_to_proceed ()

/*
 * Handle ^C in a multi-threaded environment.
 */
extern void
mach3_request_quit PARAMS ((void));
#define REQUEST_QUIT()		mach3_request_quit ()

/*
 * Startup the inferior task and suspend it after the correct code is loaded.
 */
extern void
mach3_startup_inferior PARAMS ((int));
#define STARTUP_INFERIOR(x)	mach3_startup_inferior (x)

/*
 * Determine if this is the correct thread for a breakpoint.
 */
extern int
mach3_thread_for_breakpoint PARAMS ((mach_id_t));
#define THREAD_ALLOWED_TO_BREAK(mid) mach3_thread_for_breakpoint (mid)


/*
 * Definitions needed by gdb-stop.c
 */
/*
 * Prefix for GDB message port name.
 */
#define	GDB_NAME_PREFIX	"gdb-"

/*
 * If our_message_port gets a msg with this ID,
 * GDB suspends it's inferior and enters command level.
 * (Useful at least if ^C does not work)
 */
#define GDB_MESSAGE_ID_STOP			0x41151

#endif /* _OS_MACH3_H_ */
