/* Target definitions for '386 boxes running Mach 3.0.
   Copyright (C) 1992, 1993 Free Software Foundation, Inc.

This file is part of GDB.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/*
 * HISTORY
 * $Log:	tm-i386mach3.h,v $
 * Revision 2.1.1.1  93/04/16  16:35:58  pds
 * 	Added whist markers.
 * 	[93/04/16            pds]
 * 
 */

/*
 * Include common definitions for Mach 3.0 systems.
 */
#include "os-mach3.h"

/*
 * Target specific definitions.
 */
#include <mach/i386/thread_status.h>
#include <i386/vmparam.h>	/* Is this a problem under OSF/1? - pds */

#define	KERNEL_U_ADDR	0	/* Not needed. */

/*
 * Define offsets to access CPROC stack when it does not have a kernel thread.
 */
#define MACHINE_CPROC_SP_OFFSET 20
#define MACHINE_CPROC_PC_OFFSET 16
#define MACHINE_CPROC_FP_OFFSET 12

/*
 * Thread flavors used in re-setting the T bit.
 */
#define TRACE_FLAVOR		i386_THREAD_STATE
#define TRACE_FLAVOR_SIZE	i386_THREAD_STATE_COUNT
#define TRACE_SET(state) \
  	(((struct i386_thread_state *)state)->efl |= 0x100)
#define TRACE_CLEAR(state) \
  	(((struct i386_thread_state *)state)->efl &= ~0x100)

/*
 * On Mach 3.0 this is the number of exec() system calls before
 * the task contains the debugged program. It is always 2 when
 * using the shell to start up the program.
 */
#undef  START_INFERIOR_TRAPS_EXPECTED
#define START_INFERIOR_TRAPS_EXPECTED 2

/*
 * The C-compiler puts an underscore at the front of external names
 * before giving them to the linker.
 */
#define NAMES_HAVE_UNDERSCORE

/* Sigh. There should be a file for i386 but no sysv stuff in it */
#include "tm-i386v.h"

/* I want to test this float info code. See comment in tm-i386v.h */
#undef FLOAT_INFO
#define FLOAT_INFO { i386_mach3_float_info (); }

/*
 * We use a.out format.
 */
#undef COFF_NO_LONG_FILE_NAMES

/* Address of end of stack space for MACH, see <i386/vmparam.h>
 * @@@ I don't know what is in the 5 ints...
 */
#undef  STACK_END_ADDR
#define STACK_END_ADDR	USRSTACK

/*
 * Figure out where the longjmp will land.  We expect that we have just
 * entered longjmp and that the first argument is a pointer to the jmp_buf
 * from which we extract the PC (JB_PC) where we will land.  The PC is
 * copied into ADDR and the routine returns TRUE on success.
 *
 * Under Mach 3.0 the a jmp_buf has the same layout as a struct sigcontext.
 * The PC is stored in sc_eip.
 */
#define	JB_ELEMENT_SIZE	4	/* size of target int */
#define	JB_PC		16

/*
 * This function is exported by i386-tdep.c.
 */
extern int
get_longjmp_target PARAMS ((CORE_ADDR *));

#define	GET_LONGJMP_TARGET(ADDR)	get_longjmp_target(ADDR)

/*
 * This is the layout of the top of the emulator stack used by POE, UX,
 * the Multi-server, and BSDSS.  See emulator/emul_stack.h and
 * emulator/i386/emul_vector.s for details.
 */
struct emul_stack_top {
    struct emul_stack_top *	link;
    CORE_ADDR			user_sp;
};

#ifndef	EMULATOR_STACK_SIZE
#define	EMULATOR_STACK_SIZE	(4096*4)
#endif
